//
// JTLT - a templating system for the java language
// Copyright (C) 2001-2003 Indrek Mandre <indrek@mare.ee>
// All rights reserved.
//
// See the file "AUTHORS" for the name of all copyright holders.
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program in the file "COPYING-LESSER"; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA  02111-1307  USA
//
// For more information and latest updates please see
// http://www.mare.ee/indrek/jtlt/
//

package ee.mare.indrek.jtlt;

import java.io.*;
import java.util.*;
import java.sql.*;
import ee.mare.indrek.jtlt.macros.*;

/** Html specialized templating class. */
public class HtmlTemplate extends Template {
  /** Construct a new html template from given generator. */
  public HtmlTemplate (HtmlTemplateGenerator gen)
  {
    super (gen);
  }

  /** Replace given key in the template with value in the currently
    * active block and in case of plain KEY also apply escapeSpecialChars on it. */
  public void replace (String key, String value)
  {
    do_replace (key, value, false);
  }

  /** Replace given key in the template with value in this block and all subblocks and also
    * in case of plain KEY also apply escapeSpecialChars on it.
    */
  public void nreplace (String key, String value)
  {
    do_replace (key, value, true);
  }

  private void do_replace (String key, String value, boolean is_inherited)
  {
    if ( value == null ) {
        getBase().getTemplateContext().complain ("Trying to replace null value, changing to \"\".");
        value = "";
    }

    KeyMacro.KeyValue kv = new KeyMacro.KeyValue ();
    kv.key = key;
    kv.value = value;
    kv.is_inherited = is_inherited;
    kv.do_encode = true;

    addParam ("KEY:" + key, kv);
  }

  /** The same as replace but will not HTML escape the value. */
  public void replace_raw (String key, String value)
  {
    super.replace (key, value);
  }

  /** The same as nreplace but will not HTML escape the value. */
  public void nreplace_raw (String key, String value)
  {
    super.nreplace (key, value);
  }

  /** @deprecated */
  public void replaceall_raw (String key, String value)
  {
    nreplace_raw (key, value);
  }

  /** Go through all of the rows in SQL ResultSet (using next()) and for every row lock given
    * sub and replace() all columns inside template. The column names are
    * converted to lower case. If there is a SQL NULL/null inside
    * resultset it is converted to empty string without complaints. */
  public void instance_rows (String sub_name, ResultSet rset)
  {
    try {
        ResultSetMetaData mdata = rset.getMetaData ();
        while (rset.next()) {
            lock (sub_name);
            for ( int i = 1; i <= mdata.getColumnCount(); i++ ) {
                String data = rset.getString(i);
                if ( data == null ) data = "";
                replace (mdata.getColumnName(i).toLowerCase(), data);
            }
            unlock ();
        }
    } catch (SQLException e) {
        getBase().getTemplateContext().complain ("Unable to instance_all the resultset for sub [" + sub_name + "]!", e);
    }
  }

  /** Replace a single row from the SQL ResultSet.
    * The resultset pointer must be moved pointing to the row.
    * The resultset is not modified. The column names are
    * converted to lower case. */
  public void replace (ResultSet rset)
  {
    try {
        ResultSetMetaData mdata = rset.getMetaData ();
        for ( int i = 1; i <= mdata.getColumnCount(); i++ ) {
            String data = rset.getString(i);
            if ( data == null ) data = "";
            replace (mdata.getColumnName(i).toLowerCase(), data);
        }
    } catch (SQLException e) {
        getBase().getTemplateContext().complain ("Unable to replace_row the resultset!", e);
    }
  }

  /** Translate special characters <b>&lt</b>, <b>&gt</b>, <b>"</b>, <b>'</b> and <b>&amp;</b> into HTML entities. */
  public static String encodeHTML (String str)
  {
    StringBuffer out = new StringBuffer ();
    for ( int i = 0; i < str.length(); i++ ) {
        char c = str.charAt(i);
        switch (c) {
          case '\'':
            out.append ("&#039;");
            break;
          case '"':
            out.append ("&quot;");
            break;
          case '&':
            out.append ("&amp;");
            break;
          case '>':
            out.append ("&gt;");
            break;
          case '<':
            out.append ("&lt;");
            break;
          default:
            out.append (c);
            break;
        }
    }
    return out.toString();
  }

  /** Encode-s URL arguments by replacing unrecognised characters with %XX.
    * XX is the hexcode of given character. */
  public static String encodeURL (String str)
  {
    StringBuffer out = new StringBuffer ();
    for ( int i = 0; i < str.length(); i++ ) {
        char c = str.charAt(i);
        if ( c >= 'a' && c <= 'z' || c >= 'A' && c <= 'Z' || c >= '0' && c <= '9' ) {
            out.append (c);
        } else if ( c > 255 || c == '_' || c == '.' || c == '-' ) {
            out.append (c);
        } else {
            out.append ("%");
            String hc = Integer.toHexString(c);
            if (hc.length() == 1 ) hc = "0" + hc;
            out.append(hc);
        }
    }
    return out.toString ();
  }

  /** Escape ', \, \n and \r with \ for use inside javascript strings. */
  public static String encodeJs (String str)
  {
    StringBuffer out = new StringBuffer ();
    for ( int i = 0; i < str.length(); i++ ) {
        char c = str.charAt(i);
        switch (c) {
          case '\'':
            out.append ("\\'");
            break;
          case '\\':
            out.append ("\\\\");
            break;
          case '\r':
            out.append ("\\r");
            break;
          case '\n':
            out.append ("\\n");
            break;
          default:
            out.append (c);
            break;
        }
    }
    return out.toString();
  }
}

