// 
// JTLT - a templating system for the java language
// Copyright (C) 2001-2003 Indrek Mandre <indrek@mare.ee>
// All rights reserved.
// 
// See the file "AUTHORS" for the name of all copyright holders.
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program in the file "COPYING-LESSER"; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA  02111-1307  USA
// 
// For more information and latest updates please see
// http://www.mare.ee/indrek/jtlt/
// 

package ee.mare.indrek.jtlt;

import java.util.*;
import java.io.*;
import ee.mare.indrek.jtlt.macros.KeyMacro;

/** Template class is the basic building block to generate templated content.
  */
public class Template {
  private TemplateGenerator base;
  private TemplateBlock current_ctx;

  /** Create a template using given template generator as base. */
  public Template (TemplateGenerator gen)
  {
    base = gen;
    TemplateBlock c = new TemplateBlock (this, gen, null);
    current_ctx = c;
  }

  /** Replace given key in the template with value in the currently active block. */
  public void replace (String key, String value)
  {
    if ( value == null ) {
        getBase().getTemplateContext().complain ("Trying to replace null value, changing to \"\".");
        value = "";
    }

    KeyMacro.KeyValue kv = new KeyMacro.KeyValue ();
    kv.key = key;
    kv.value = value;

    addParam ("KEY:" + key, kv);
  }

  /** Replace given key in the template with value in this block and all subblocks.
    * Actually the subblocks query the parent block for key-s and so use this
    * value. */
  public void nreplace (String key, String value)
  {
    if ( value == null ) {
        getBase().getTemplateContext().complain ("Trying to replace null value, changing to \"\".");
        value = "";
    }

    KeyMacro.KeyValue kv = new KeyMacro.KeyValue ();
    kv.key = key;
    kv.value = value;
    kv.is_inherited = true;

    addParam ("KEY:" + key, kv);
  }

  /** @deprecated */
  public void replaceall (String key, String value)
  {
    nreplace (key, value);
  }

  /** Lock given sub and do tlt.replace (map).
    * @see #replace(Map)*/
  public void instance (String sub_name, Map map)
  {
    lock (sub_name);
    replace (map);
    unlock ();
  }

  /** Replace all keys-values inside the map.
    * So the map must contain String:String associations. */
  public void replace (Map map)
  {
    Iterator it = map.entrySet().iterator();
    while ( it.hasNext() ) {
        Map.Entry me = (Map.Entry)it.next();
        replace ((String)me.getKey(), (String)me.getValue());
    }
  }

  private void add_param (String identifier, Object val)
  {
    current_ctx.set_param (identifier, val);
  }

  /** Set macro parameters for given block and sub-blocks to val.
    * Given object shadows here and in any subblocks parameters defined
    * with same name in parent blocks. */
  public void addParam (String key, Object val)
  {
    add_param (key, val);
  }

  /** Lock and unlock given block name in one call. */
  public boolean instance (String sub_name)
  {
    boolean ret = lock(sub_name);
    if ( ret ) unlock ();
    return ret;
  }

  /** Lock and create another block of given subtemplate for replacements.
    * The block is placed at the back of previously created that named blocks.
    * If the locking fails a complaint will be written.
    * @return whether a subtemplate with given name was found and locking succeeded
    * @see TemplateContext#setComplainTo
    */
  public boolean lock (String sub_name)
  {
    return lock_back (sub_name);
  }

  /** Lock and create another block of given subtemplate for replacements,
    * placing the block at the front of other blocks previously created. */
  public boolean lock_front (String sub_name)
  {
    return do_lock (sub_name, false);
  }

  /** Lock and create another block of given subtemplate for replacements,
    * placing the block at the back of other blocks previously created. */
  public boolean lock_back (String sub_name)
  {
    return do_lock (sub_name, true);
  }

  /** To find out whether a given subblock exists. */
  public boolean exists (String name)
  {
    return current_ctx.generator.getGenerator (name) != null;
  }

  private boolean do_lock (String name, boolean at_back)
  {
    TemplateGenerator sub = current_ctx.generator.getGenerator (name);
    if ( sub == null ) {
        getBase().getTemplateContext().complain ("Trying to lock a nonexistent block '" + name +"'!");
        return false;
    }

    TemplateBlock nc = new TemplateBlock (this, sub, current_ctx);
    current_ctx.add_sub (nc, at_back);
    current_ctx = nc;

    return true;
  }

  /** Unlock previously locked block of subtemplate moving up in scope.
    * If you try to unlock the top block a complain will be written
    * and the function will just return. */
  public void unlock ()
  {
    if ( current_ctx.parent != null )
        current_ctx = current_ctx.parent;
    else
        getBase().getTemplateContext().complain ("Trying to unlock a block while in the top block!");
  }

  /** Get the base generator template.
    */
  public TemplateGenerator getBase()
  {
    return base;
  }

  /** Convert the template to string. */
  public String toString ()
  {
    try {
      StringWriter w = new StringWriter ();
      current_ctx.append (w);
      return w.toString();
    } catch (Exception e) {
      return "";
    }
  }

  /** Output the template contents. */
  public void output (Writer out) throws IOException
  {
    current_ctx.append (out);
  }

  private class Pair {
    Object obj1;
    Object obj2;

    Pair (Object o1, Object o2)
    {
      obj1 = o1;
      obj2 = o2;
    }

    public int hashCode ()
    {
      return obj1.hashCode() + obj2.hashCode();
    }

    public boolean equals (Object obj)
    {
      if ( ! (obj instanceof Pair) ) return false;
      Pair p = (Pair)obj;
      return obj1.equals (p.obj1) && obj2.equals (p.obj2);
    }
  };

  // Now the theory goes - it is better to construct
  // one large hash rather than 100 little ones.
  Map content_map = new HashMap (128);

  void set_content (Object k1, Object k2, Object val)
  {
    content_map.put (new Pair (k1, k2), val);
  }

  Object get_content (Object k1, Object k2)
  {
    return content_map.get (new Pair (k1, k2));
  }
}

