// 
// JTLT - a templating system for the java language
// Copyright (C) 2001-2003 Indrek Mandre <indrek@mare.ee>
// All rights reserved.
// 
// See the file "AUTHORS" for the name of all copyright holders.
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program in the file "COPYING-LESSER"; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA  02111-1307  USA
// 
// For more information and latest updates please see
// http://www.mare.ee/indrek/jtlt/
// 

package ee.mare.indrek.jtlt;

import java.util.*;
import java.io.*;
import ee.mare.indrek.jtlt.macros.*;

/**
  * TemplateContext represents the settings and parameters that are used
  * with working templates derived from given context. Usually you'll
  * be using the default context. The default
  * parameters are: startTag is &lt;!--, endTag is --&gt;.
  * Also only one macro that handles the KEY-s is defined. */
public class TemplateContext {

  private String start_tag = "<!--";
  private String end_tag = "-->";
  Map shortcut_tags = new HashMap ();

  private Map macros = new HashMap ();

  /** Construct a new TemplateContext. These are used to create instances of templates
    * with given parameters (start/end tag), tempalte macros, filters, etc.
    */
  public TemplateContext ()
  {
    registerMacro (new KeyMacro());
  }

  /** Get the starting string for template directives.
    * @see #setStartTag
    */
  public String getStartTag () { return start_tag; }
  /** Set the starting tag for template directives.
    * @see #getStartTag
    */
  public void setStartTag (String ss) { start_tag = ss; }

  /** Set a shortcut tags for macro. */
  public void setShortcutTags (String macro, String start, String end)
  { shortcut_tags.put(macro, new String[] {start, end}); }

  /** Clear shortcut tags for macro. */
  public void clearShortcutTags (String macro)
  { shortcut_tags.remove(macro); }

  /** Get the ending string for template directives.
    * @see #setEndTag
    */
  public String getEndTag () { return end_tag; }
  /** Set the ending tag for template directives.
    * @see #getEndTag
    */
  public void setEndTag (String ss) { end_tag = ss; }

  /** Register a macro.
    * @see #findMacro
    */
  public void registerMacro (TemplateMacro m)
  {
    macros.put (m.getName(), m);
  }

  /** Find a registered macro with given name.
    * @see #registerMacro
    */
  public TemplateMacro findMacro (String name)
  {
    return (TemplateMacro)macros.get(name);
  }

  private PrintStream complain_out = System.err;

  /** Set the stream where to complain when unexpected errors happen.
    * Complains will be mostly written when you try to lock a block
    * that does not exist. Can be set to null to disable complaints.
    * By default set to System.err.
    * @return the previously assigned complain stream */
  public PrintStream setComplainTo (PrintStream out)
  {
    PrintStream old = complain_out;
    complain_out = out;
    return old;
  }

  /** This method is called by templates when an error happens at runtime.
    * Currently it only prints out the error message and a current stack trace to
    * the stream specified by {@link #setComplainTo}.
    */
  public void complain (String err)
  {
    if ( complain_out != null ) {
        try {
          throw new TemplateException (err);
        } catch (Exception e) {
          e.printStackTrace (complain_out);
        }
    }
  }

  /** Complain by getting an additional exception to complain about. */
  public void complain (String err, Exception ex)
  {
    if ( complain_out != null ) {
        try {
          throw new TemplateException (err);
        } catch (Exception e) {
          ex.printStackTrace (complain_out);
          e.printStackTrace (complain_out);
        }
    }
  }

  private String charset = "ISO-8859-1";
  /** Get the charset used when opening template files for reading.
    * The default charset is ISO-8859-1. */
  public String getCharset ()
  {
    return charset;
  }

  /** Set the charset that is used when opening template files. */
  public void setCharset (String charset)
  {
    this.charset = charset;
  }
}

