// 
// JTLT - a templating system for the java language
// Copyright (C) 2001-2003 Indrek Mandre <indrek@mare.ee>
// All rights reserved.
// 
// See the file "AUTHORS" for the name of all copyright holders.
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program in the file "COPYING-LESSER"; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA  02111-1307  USA
// 
// For more information and latest updates please see
// http://www.mare.ee/indrek/jtlt/
// 

package ee.mare.indrek.jtlt;

import java.util.*;
import java.io.*;

/** Class for generating instances of templates from one template file.
  * This class represents the parsed template structure that can be used by multiple Template classes.
  * The class implements TemplateParams and passes itself to parsetime macros at parse time. */
public class TemplateGenerator implements TemplateParams {
  
  private TemplateContext context;
  private String name = "";
  private List templateParts;
  private List subs = new LinkedList ();
  private Map sub2tlt = new HashMap ();

  /** Construct a TemplateGenerator from Reader 'in' using the default TemplateContext. */
  public TemplateGenerator (Reader in) throws IOException, TemplateException
  {
    this (in, default_ctx);
  }

  /** Construct a TemplateGenerator from path using the default TemplateContext. */
  public TemplateGenerator (String path) throws IOException, TemplateException
  {
    this (path, default_ctx);
  }

  /** Construct a TemplateGenerator from path using given TemplateContext. */
  public TemplateGenerator (String path, TemplateContext ctx) throws IOException, TemplateException
  {

    Reader fr = new InputStreamReader (new FileInputStream (path), ctx.getCharset());
    init_from_reader (new BufferedReader (fr), ctx);
    fr.close();
  }

  /** Construct a TemplateGenerator from Reader 'in' and associated with context. */
  public TemplateGenerator (Reader in, TemplateContext ctx) throws IOException, TemplateException
  {
    init_from_reader (in, ctx);
  }

  private void init_from_reader (Reader in, TemplateContext ctx) throws IOException, TemplateException
  {
    context = ctx;
    StringBuffer buf = new StringBuffer ();

    char[] b = new char[4096];
    int read = 0;
    while ( (read = in.read(b)) > 0)
        buf.append (b, 0, read);

    init (buf.toString(), 1);
  }

  private TemplateGenerator (String n, String str, int line, TemplateContext gen) throws TemplateException
  {
    name = n;
    context = gen;
    init (str, line);
  }

  private String start_tag;
  private String end_tag;

  private void init (String str, int line) throws TemplateException
  {
    start_tag = context.getStartTag ();
    end_tag = context.getEndTag ();

    templateParts = parse (str, line, true);
  }

  LinkedList parse (String str, int line, boolean allow_subs) throws TemplateException
  {
//    System.err.println ("parse: [" + str + "], line = " + line + ", allow_subs = " + allow_subs);
    LinkedList parts = new LinkedList ();
    int s = 0;

    StringBuffer acc = new StringBuffer ();

    while ( s != str.length() ) {
        String macro_name = null;
        String macro_args = null;
        String stag = start_tag;
        String etag = end_tag;

        int i = str.indexOf (start_tag, s);
        for ( Iterator it = context.shortcut_tags.entrySet().iterator(); it.hasNext(); ) {
            Map.Entry entry = (Map.Entry)it.next();
            String m = (String)entry.getKey();
            String[] tags = (String[])entry.getValue();
            int j = str.indexOf (tags[0], s);
            if ( i == -1 || j >= 0 && j < i ) {
                macro_name = m;
                stag = tags[0];
                etag = tags[1];
                i = j;
            }
        }

        if ( i == -1 ) i = str.length ();

        // It seems we have a string.
        if ( s != i ) {
            String t = str.substring (s, i);
            acc.append (t);
//            parts.add (t);
            line += calcLines (t);

            s = i;
            if ( s == str.length() ) break;
        }

        // Find macro name and arguments (if not already known)
        int e = findMacroEnd (str, etag, i + stag.length());
        if ( e == -1 )
            throw new TemplateException ("Parse error, no valid macro end tag found [" + line + "]");

        if ( macro_name == null ) {
            int colon = str.indexOf (":", i);

            if ( colon == -1 || colon == i + stag.length() || colon > e ) {
                macro_name = str.substring (i + stag.length(), e);
                macro_args = "";
            } else {
                macro_name = str.substring (i + stag.length(), colon);
                macro_args = str.substring (colon + 1, e);
            }
        } else {
            macro_args = str.substring(i + stag.length(), e);
        }

        List margs = parse (macro_args, line, false);

        String real_macro_args = null;
        if ( margs.size() == 0 ) {
            real_macro_args = "";
        } else if (margs.size() == 1 && margs.get(0) instanceof String ) {
            real_macro_args = (String)margs.get(0);
        }

        // Dealing with a sub-template
        if ( macro_name.equals("SUB") ) {
            String sub_name = real_macro_args;
            if ( sub_name == null )
                throw new TemplateException ("Parse error, sub template name must be resolvable" +
                    " at parse time at line [" + line + "]");

            String end_name = stag + "END:" + sub_name + etag;
            int e2 = str.indexOf (end_name, e);
            if ( e2 == -1 )
                throw new TemplateException ("Parse error, no end tag found for subtemplate '" + sub_name +
                    "' starting at line [" + line + "]");

            if ( sub2tlt.get(sub_name) != null )
                throw new TemplateException ("Parse error, sub block '" + sub_name +
                    "' at line " + line + " is already existing in this block level");

            line += calcLines (sub_name); // Might be a weird person using newlines in sub name.

            String sub = str.substring (e + etag.length(), e2);

            if ( allow_subs ) {
                TemplateGenerator stlt = new TemplateGenerator (sub_name, sub, line, context);
                sub2tlt.put (sub_name, stlt);

                if ( acc.length() > 0 ) { // Get away with the accumulated string
                    parts.add (acc.toString());
                    acc.setLength (0);
                }

                parts.add (stlt);
                subs.add (stlt);
            } else {
                context.complain ("SUB elements not allowed at non-parsetime macros or in macro arguments!");
            }

            line += calcLines (sub);
            line += calcLines (sub_name); // Also for end tag.

            s = e2 + end_name.length();
            continue;
        }

        TemplateMacro macro = context.findMacro (macro_name);
        if ( macro != null ) {
            if ( macro.isParsetime () && real_macro_args != null ) {
                String add = macro.process (this, macro_name, real_macro_args);
                if ( macro.isRecursive () ) {
                    str = str.substring (0, s) + add + str.substring (e + etag.length());
                    continue;
                }
                acc.append (add);
            } else {
                MacroArgs m = new MacroArgs();
                m.macro = macro;
                m.name = macro_name;
                m.args = margs;

                if ( acc.length() > 0 ) { // Get away with the accumulated string
                    parts.add (acc.toString());
                    acc.setLength (0);
                }

                parts.add (m);
            }
        } else if ( !macro_name.startsWith(" ") ) {
            context.complain ("Unknown macro [" + macro_name + "] at line " + line);
        }

        line += calcLines (macro_name);
        line += calcLines (macro_args);

        s = e + etag.length();
    }

    if ( acc.length() > 0 ) { // Get away with the accumulated string
        parts.add (acc.toString());
        acc.setLength (0);
    }

    return parts;
  }

  private int findMacroEnd (String str, String search_end, int s)
  {
    int s1 = str.indexOf (search_end, s);

    int s2 = str.indexOf (start_tag, s);
    String stag = start_tag;
    String etag = end_tag;

    for ( Iterator it = context.shortcut_tags.entrySet().iterator(); it.hasNext(); ) {
        Map.Entry entry = (Map.Entry)it.next();
        String m = (String)entry.getKey();
        String[] tags = (String[])entry.getValue();
        int i = str.indexOf (tags[0], s);
        if ( s2 == -1 || i >= 0 && i < s2 ) {
            s2 = i;
            stag = tags[0];
            etag = tags[1];
        }
    }

    if ( s1 == -1 && s2 == -1 ) return -1;
    if ( s2 == -1 || s1 != -1 && s1 < s2 ) return s1;

    // backup plan, macro argument is macro, find its end
    s = findMacroEnd (str, etag, s2 + stag.length()) + etag.length();

    return findMacroEnd (str, search_end, s);
  }

  private static int calcLines (String l)
  {
    int ret = 0;
    int s = 0;
    while ( (s = l.indexOf ('\n', s)) != -1 )
      {
        ret++;
        s++;
      }

    return ret;
  }

  /** Create a new template instance from this template generator. */
  public Template createTemplate ()
  {
    return new Template (this);
  }

  /** Get a template generator for subtemplate with given name. */
  TemplateGenerator getGenerator (String name)
  {
    return (TemplateGenerator)sub2tlt.get(name);
  }
  
  List getParts ()
  {
    return templateParts;
  }

  public Object findParam (String ident, boolean search_parent)
  {
    return null;
  }

  public TemplateContext getTemplateContext ()
  {
    return context;
  }

  private static TemplateContext default_ctx = new TemplateContext ();
}

